<?php

namespace Drupal\commerce_shipping_product_exclude\Plugin\Commerce\Condition;

use Drupal\commerce\Plugin\Commerce\Condition\ConditionBase;
use Drupal\commerce\Plugin\Commerce\Condition\ParentEntityAwareInterface;
use Drupal\commerce\Plugin\Commerce\Condition\ParentEntityAwareTrait;
use Drupal\commerce_product\Entity\ProductVariationInterface;
use Drupal\commerce_shipping_product_exclude\Plugin\Field\FieldType\ExcludeShippingMethodItem;
use Drupal\Core\Entity\EntityInterface;

/**
 * Provides the weight condition for shipments.
 *
 * @CommerceCondition(
 *   id = "exclude_from_shipping",
 *   label = @Translation("Allow to exclude From Shipping"),
 *   description = @Translation("Allows the targeted shipping method, to not appear, if ANY product's / product variation's 'Exclude Shipping Method' field in the order references it."),
 *   category = @Translation("Commerce Order"),
 *   entity_type = "commerce_order",
 * )
 */
class ExcludeFromShipping extends ConditionBase implements ParentEntityAwareInterface {
  use ParentEntityAwareTrait;

  /**
   * {@inheritdoc}
   */
  public function evaluate(EntityInterface $entity) {
    $this->assertEntity($entity);
    /** @var \Drupal\commerce_order\Entity\OrderInterface $order */
    $order = $entity;

    foreach ($order->getItems() as $orderItem) {
      $productVariation = $orderItem->getPurchasedEntity();
      if (!($productVariation instanceof ProductVariationInterface)) {
        // For some reason, the purchased entity is not a product variation.
        // Skip it:
        continue;
      }

      // Check if the product variation has an 'exclude_shipping_method' field,
      // which isn't empty:
      foreach ($productVariation->getFields() as $field) {
        if ($field->getFieldDefinition()->getType() === ExcludeShippingMethodItem::FIELD_ID) {
          if (!empty($field->getValue())) {
            // Check if the current shipping method should be excluded on
            // variation level:
            $excludedShippingMethods = $field->getValue();
            foreach ($excludedShippingMethods as $excludedShippingMethod) {
              if ($excludedShippingMethod['target_id'] === $this->parentEntity->id()) {
                // The shipping method was found and should be excluded, the
                // evaluation fails:
                return FALSE;
              }
            }
          }
        }
      }

      // If the exclude shipping field is still empty, we check if the parent
      // product has the field:
      $product = $productVariation->getProduct();
      foreach ($product->getFields() as $field) {
        if ($field->getFieldDefinition()->getType() === ExcludeShippingMethodItem::FIELD_ID) {
          if (!empty($field->getValue())) {
            // Check if the current shipping method should be excluded on
            // product level:
            $excludedShippingMethods = $field->getValue();
            foreach ($excludedShippingMethods as $excludedShippingMethod) {
              if ($excludedShippingMethod['target_id'] === $this->parentEntity->id()) {
                // The shipping method was found and should be excluded, the
                // evaluation fails:
                return FALSE;
              }
            }
          }
        }
      }
    }
    // This shipping method is not excluded, let the evaluation succeed:
    return TRUE;
  }

}
