<?php

declare(strict_types=1);

namespace Drupal\commerce_shipping_product_exclude\Plugin\Field\FieldType;

use Drupal\Core\Field\Attribute\FieldType;
use Drupal\Core\Field\EntityReferenceFieldItemList;
use Drupal\Core\Field\Plugin\Field\FieldType\EntityReferenceItem;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;

/**
 * Defines the 'commerce_shipping_product_exclude_shipping_method' field type.
 */
#[FieldType(
   id : self::FIELD_ID,
   label : new TranslatableMarkup("Exclude Shipping Method"),
   category : "commerce",
   description : new TranslatableMarkup("Exclude shipping methods used for this fieldable entity."),
   default_widget : "options_buttons",
   default_formatter : "entity_reference_label",
   list_class: EntityReferenceFieldItemList::class,
 )]
class ExcludeShippingMethodItem extends EntityReferenceItem {

  const FIELD_ID = 'commerce_shipping_product_exclude_shipping_method';

  /**
   * {@inheritdoc}
   */
  public static function defaultStorageSettings() {
    // We don't want to return the parent's default storage settings, because
    // they already implement the 'target_type':
    return [
      'target_type' => 'commerce_shipping_method',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function storageSettingsForm(array &$form, FormStateInterface $form_state, $has_data) {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public static function getPreconfiguredOptions() {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function getSettableOptions(?AccountInterface $account = NULL) {
    // Get all selectable options:
    $selectableOptions = parent::getSettableOptions($account);
    // Get all available and enabled shipping methods:
    $shippingMethodStorage = \Drupal::entityTypeManager()->getStorage('commerce_shipping_method');
    $query = $shippingMethodStorage->getQuery();
    $shippingMethodIds = $query->accessCheck(FALSE)->condition('status', TRUE)->sort('weight')->execute();
    $shippingMethods = $shippingMethodStorage->loadMultiple($shippingMethodIds);

    $filteredSelectableOptions = [];
    // Filter out shipping methods that don't have the 'exclude_from_shipping'
    // condition enabled:
    foreach ($shippingMethods as $shippingMethod) {
      $excludeFromShippingCondition = array_filter($shippingMethod->getConditions(), function ($condition) {
        return $condition->getPluginId() === 'exclude_from_shipping';
      });
      // The shipping method does not have "exclude from shipping" enabled.
      // Continue to the next one:
      if (empty($excludeFromShippingCondition)) {
        continue;
      }
      // The shipping method does have "exclude from shipping" enabled,
      // get the first one (there should be only one anyway):
      $excludeFromShippingCondition = reset($excludeFromShippingCondition);
      // Build the new selectable options based on the shipping methods that
      // have the 'exclude_from_shipping' condition enabled:
      $filteredSelectableOptions[$shippingMethod->id()] = $selectableOptions[$shippingMethod->id()];
    }
    return $filteredSelectableOptions;
  }

}
